==> nextcode-sdk-1.12.1.tar.gz: nextcode-sdk-1.12.1/nextcode/services/queryserver/service.py <==
"""
Service class
------------------
Service object for interfacing with the GOR Query Server.

This class instance is used to communicate with a RESTFul service. Use the `execute`
methods to get started.

"""
import json
import logging
import os
from typing import Dict, Tuple, Sequence, List, Optional, Union, Any

from .result import Result
from ...services import BaseService
from ...exceptions import ServerError
from ...client import Client
from ..query.exceptions import QueryError, MissingRelations

from ..query.utils import extract_virtual_relations
import nextcode

SERVICE_PATH = "queryserver"

log = logging.getLogger(__name__)


class Service(BaseService):
    """
    A connection to the query server
    """

    def __init__(self, client: Client, *args, **kwargs) -> None:
        super(Service, self).__init__(client, SERVICE_PATH, *args, **kwargs)
        self.metadata = {"client": f"nextcode-python-sdk/{nextcode.__version__}"}
        self.project = (
            kwargs.get("project")
            or os.environ.get("GOR_API_PROJECT")
            or client.profile.project
        )

    def _check_project(self):
        """
        Raise an exception if there is no project specified

        :raises: QueryError
        """
        if not self.project:
            raise QueryError("No project specified")

    def set_project(self, project: str, persist: bool = True) -> None:
        """
        Set the current project for all subsequent queries.

        :project: The project name
        :persist: Persist the project name into the current profile

        """
        self.project = project
        if persist:
            self.client.profile.project = self.project

    def execute(
        self,
        query: str,
        relations: Optional[List[Dict]] = None,
        job_type: Optional[str] = None,
        gzip: Optional[bool] = False,
        **kw,
    ):  # -> SynchronousQuery:
        """
        Execute a gor statement on the server synchronously.

        :param query: gor query string
        :param relations: virtual relations to include with the query
        :param job_type: Optional job type for routing purposes
        :param gzip:Optional if result should be gzipped.

        :raises: :exc:`~exceptions.ServerError`, :exc:`~exceptions.MissingRelations`

        Optional keyword arguments in the form `name=data` (where data is a tsv string) are converted into virtual relations if relations
        is not explicitly passed in and added to the relations set by the relations parameter. If the relations parameter is set it
        should be a list of dictionaries with {"name": "relation-name", "data": "<tsv string>"}
        """
        self._check_project()

        payload_relations = extract_virtual_relations(kw, relations)

        payload: Dict[str, Optional[Union[int, str, List[Any], Dict]]] = {
            "project": self.project,
            "query": query,
            "user": "python-sdk",
            "virtualRelations": payload_relations,
            "routingKey": job_type or "default",
            "useGzip": str(gzip).lower(),
            "sendTerm": True,
            "sendProgress": True,
            "sendAlive": True
        }

        url = self.session.url_from_endpoint("query")

        try:
            resp = self.session.post(url,
                                     json=payload,
                                     stream=True,
                                     headers={"Content-Type": "application/json",
                                              "Accept-Encoding": "chunked",
                                              "Accept": "application/octet-stream"})
            resp.raise_for_status()
        except ServerError as ex:
            json_ex = json.loads(ex.message[ex.message.find('{'):ex.message.rfind('}')+1])
            if "errorType" in json_ex and json_ex["errorType"] == "GorMissingRelationException":
                raise MissingRelations(
                    [r for r in json_ex["uri"].split(',')]
                )
            else:
                raise ServerError("{} in command: {}\n{}".format(
                    json_ex["errorType"] if "errorType" in json_ex else "Unknown error",
                    json_ex["command"] if "command" in json_ex else "unknown command",
                    json_ex["message"] if "message" in json_ex else "No message"))

        return Result(resp, gzip)

==> nextcode-sdk-1.17.0.tar.gz: nextcode-sdk-1.17.0/nextcode/services/queryserver/service.py <==
"""
Service class
------------------
Service object for interfacing with the GOR Query Server.

This class instance is used to communicate with a RESTFul service. Use the `execute`
methods to get started.

"""
import json
import logging
import os
from typing import Dict, Tuple, Sequence, List, Optional, Union, Any

from .result import Result
from ...services import BaseService
from ...exceptions import ServerError
from ...client import Client
from ..query.exceptions import QueryError, MissingRelations

from ..query.utils import extract_virtual_relations
import nextcode

SERVICE_PATH = "queryserver"

log = logging.getLogger(__name__)


class Service(BaseService):
    """
    A connection to the query server
    """

    def __init__(self, client: Client, *args, **kwargs) -> None:
        super(Service, self).__init__(client, SERVICE_PATH, *args, **kwargs)
        self.metadata = {"client": f"nextcode-python-sdk/{nextcode.__version__}"}
        self.project = (
            kwargs.get("project")
            or os.environ.get("GOR_API_PROJECT")
            or client.profile.project
        )

    def _check_project(self):
        """
        Raise an exception if there is no project specified

        :raises: QueryError
        """
        if not self.project:
            raise QueryError("No project specified")

    def set_project(self, project: str, persist: bool = True) -> None:
        """
        Set the current project for all subsequent queries.

        :project: The project name
        :persist: Persist the project name into the current profile

        """
        self.project = project
        if persist:
            self.client.profile.project = self.project

    def execute(
        self,
        query: str,
        relations: Optional[List[Dict]] = None,
        job_type: Optional[str] = None,
        gzip: Optional[bool] = False,
        **kw,
    ):  # -> SynchronousQuery:
        """
        Execute a gor statement on the server synchronously.

        :param query: gor query string
        :param relations: virtual relations to include with the query
        :param job_type: Optional job type for routing purposes
        :param gzip:Optional if result should be gzipped.

        :raises: :exc:`~exceptions.ServerError`, :exc:`~exceptions.MissingRelations`

        Optional keyword arguments in the form `name=data` (where data is a tsv string) are converted into virtual relations if relations
        is not explicitly passed in and added to the relations set by the relations parameter. If the relations parameter is set it
        should be a list of dictionaries with {"name": "relation-name", "data": "<tsv string>"}
        """
        self._check_project()

        payload_relations = extract_virtual_relations(kw, relations)

        payload: Dict[str, Optional[Union[int, str, List[Any], Dict]]] = {
            "project": self.project,
            "query": query,
            "user": "python-sdk",
            "virtualRelations": payload_relations,
            "routingKey": job_type or "default",
            "useGzip": str(gzip).lower(),
            "sendTerm": True,
            "sendProgress": True,
            "sendAlive": True
        }

        url = self.session.url_from_endpoint("query")

        try:
            resp = self.session.post(url,
                                     json=payload,
                                     stream=True,
                                     headers={"Content-Type": "application/json",
                                              "Accept-Encoding": "chunked",
                                              "Accept": "application/octet-stream"})
            resp.raise_for_status()
        except ServerError as ex:
            if ex.message.startswith("#>") and ex.message.find('{') > -1:
                # Assume this is an exception from Query Server with embedded json.
                json_ex = json.loads(ex.message[ex.message.find('{'):ex.message.rfind('}')+1])
                if "errorType" in json_ex and json_ex["errorType"] == "GorMissingRelationException":
                    raise MissingRelations(
                        [r for r in json_ex["uri"].split(',')]
                    )
                else:
                    raise ServerError(json_ex["gorMessage"])
            else:
                raise ex

        return Result(resp, gzip)
